/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.jdbc;

import java.net.*;
import java.sql.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import com.adobe.gl.runtime6.utils.*;

/**
 * GL ContentSourceWrapper for JDBC ResultSet.
 *
 * Because we don't count on scrollable results (JDBC 1.0 limitation),
 * we currently grab up all the data in the ResultSet and store it in
 * memory.  An implementation closer to the ASP version would grab only
 * rows up to the high-water mark maintained by this.IX (available
 * externally as AbsolutePosition()).  Another way to go would be to
 * limit the query to produce a ResultSet of the required size.
 */
public class CSWResultSet
	implements ContentSourceWrapper
{
	/*
	 * -- Constructors
	 */
	private CSWResultSet(ResultSet rs, String source, String sql,
		String pkey, GLContext gl)
	{
		this.gl = gl;
		gl.registerCSW(this);
		this.pkey = GLUtils.split(",", pkey);
		this.source = source;
		this.sql = sql;
		this.ix = 1;

		/* Extract column labels and count. */
		try {
			ResultSetMetaData mdata = rs.getMetaData();
			this.ncol = mdata.getColumnCount();
			this.colnums = new Hashtable(this.ncol);
			this.colname = new String[1+this.ncol];
			this.coltype = new int[1+this.ncol];
			this.coltypename = new String[1+this.ncol];
			for (int j = 1; j <= this.ncol; ++j) {
				this.colname[j] = mdata.getColumnLabel(j);
				this.colnums.put(
					mdata.getColumnLabel(j).toLowerCase(),
					new Integer(j));
				this.coltype[j] = mdata.getColumnType(j);
				this.coltypename[j] =
					mdata.getColumnTypeName(j);
			}
			Vector v = new Vector();
			v.addElement(null);
			try {
				int nr = 0;
				for (; rs.next(); ++nr) {
					Object[] w = new Object[1+this.ncol];
					for (int j = 1; j <= this.ncol; ++j)
						w[j] = rs.getObject(j);
					v.addElement(w);
				}
				this.blksize = 1;
				this.nrow = nr;
			}
			catch (SQLException se) {
				gl.runtimeDebugMessage("Caught " + se);
				v.setSize(1);
				this.blksize = this.nrow = 0;
			}
			this.a = new Object[v.size()][1+this.ncol];
			v.copyInto(this.a);
		}
		catch (SQLException ae) {
			this.ncol = 0;
			this.colnums = null;
		}
	}

	/**
	 * Obtain a JDBC result set from the database SOURCE, using
	 * primary key PKEY, and query SQL.
	 * @deprecated See {@link #wrapJDBCResult(GLContext, String, String,
	 *		String, String, String)}
	 */
	public static CSWResultSet
	wrapJDBCResultSet(GLContext gl, String source, String sql,
		String blocktype, String pkey)
		throws GLException
	{
		return wrapJDBCResultSet(gl, source, sql, blocktype,
			pkey, "NoName");
	}

	/**
	 * Obtain a JDBC result set from the database SOURCE, using
	 * primary key PKEY, query SQL, and name NAME.
	 */
	public static CSWResultSet
	wrapJDBCResultSet(GLContext gl, String source, String sql,
		String blocktype, String pkey, String name)
		throws GLException
	{
		Connection db = null;
		Statement st = null;
		ResultSet rs = null;
		CSWResultSet wrap = null;
		String msg = "";
		String xname = "wrapJDBCResultSet";

		try {
			if ("".equals(pkey))
				pkey = getPrimaryKey(gl, source,
					GLJdbc.getTableName(sql));
			db = GLJdbc.getConnection(gl, source);
			st = db.createStatement();
			gl.runtimeDebugMessage(
				GLUtils.cat(xname, ": ", sql, ""));
			rs = st.executeQuery(sql);
			wrap = new CSWResultSet(rs, source, sql, pkey, gl);
			wrap.name = name;
			wrap.initPosition(blocktype);
			wrap.initErrors(gl);
		}
		catch (Exception e) {
			try {
				if (wrap != null) {
					wrap.Close();
					wrap = null;
				}
			}
			catch (Exception e2) {
			}

			StringBuffer ts = new StringBuffer();
			ts.append("\r\n");
			GLDebug.reportException(e, ts);
			msg = ts.toString();
			ts = null;
			gl.exception = e;
		}
		finally {
			try {
				if (rs != null)
					rs.close();
				if (st != null)
					st.close();
				if (db != null)
					db.close();
			}
			catch (Exception e2) {
			}
		}

		if (wrap==null)
			throw new GLException(xname + msg);
		if (gl.runtimeDebug)
			gl.runtimeDebugMessage(wrap.toString());
		return wrap;
	}

	/*
	 * -- Implement the ContentSourceWrapper interface.
	 */

	public int
	BlockSize()
	{
		return blksize;
	}

	public void
	Close() {
	}

	/**
	 * Returns a Hashtable memoizing this csw.
	 */
	public Hashtable
	GetGenerator(boolean allRecords)
		throws GLException
	{
		Hashtable a = new Hashtable();
		a.put("source", source);
		a.put("sql", sql);
		a.put("pkey", GLUtils.join(",", pkey));
		StringBuffer ts = new StringBuffer();
		if (newRecord) {
			String k = "[" + ix + "]";
			Hashtable b = new Hashtable();
			b.put(NEW_RECORD, NEW_RECORD);
			a.put(k, b);
			ts.append(k);
		}
		{
			String sep = "";
			int ixsave = ix;
			for (MoveTo(FirstRecordOnPage()); !EOB(); Move(1)) {
				String k = "[" + ix + "]";
				a.put(k, getWhere());
				ts.append(sep).append(k);
				sep = ",";
			}
			for (int i = 0; i < indices.size(); ++i) {
				int ix = ((Integer) indices.get(i)).intValue();
				MoveTo(ix);
				String k = "[" + ix + "]";
				if (a.containsKey(k))
					continue;
				a.put(k, getWhere());
				ts.append(sep).append(k);
				sep = ",";
			}
			indices.clear();
			MoveTo(ixsave);
		}
		a.put("key", ts.toString());
		return a;
	}

	public void Move(int delta) {
		ix += delta;
	}
	public void MoveFirst() {
		ix = 1;
	}
	public void MoveNext() {
		++ix;
	}
	public void MoveTo(int pos) {
		ix = pos;
	}

	/**
	 * Navigates to the position indicated by KEYVAL (i.e., the leftmost
	 * record whose values match those in keyval), or EOF if no such
	 * record is present.
	 */
	public void
	MoveToKey(Hashtable keyval)
		throws GLException
	{
		/*
		 * Handle special cases: $copyRecord, $lastRecord, $newRecord.
		 */
		boolean isCopy = false, isLast = false;
		if (keyval.get("$newRecord") != null
		 || (isCopy = keyval.get("$copyRecord") != null)
		 || (isLast = keyval.get("$lastRecord") != null))
		{
			if (isLast) {
				int bs = blksize > 1 ? blksize : 1,
					shift0 = nrow % bs,
					shift = shift0 != 0 ? shift0 : bs;
				MoveTo(nrow-shift+1);
			}
			else {
				MoveTo(1+nrow);		/* past EOF */
				this.newRecord = true;
				if (isCopy)
					this.copyRecord = 1;
			}
			return;
		}

		String[] keynames = GLUtils.hkeys(keyval);
	Rowloop:
		for (MoveFirst(); !EOF(); Move(1)) {
			for (int j = 0; j < keynames.length; ++j) {
				String kn = keynames[j];
				if (!GLContext.eqstr(Value(kn),
					(String) keyval.get(kn)))
				{
					continue Rowloop;
				}
			}
			return;
		}
	}

	public ContentSourceWrapper
	NestedCSW(String name)
		throws GLException
	{
		throw new GLException("CSWResultSet: I don't do NestedCSW");
	}

	/**
	 * We're off the end of our block.
	 */
	public boolean EOB() {
		return EOF() || ix > lastRec;
	}

	/**
	 * True iff we've run off the end of the results.
	 */
	public boolean EOF() {
		return this.nrow < this.ix;
	}

	public String
	Error(String key) {
		String errmsg = "";
		if (errors != null) {
			errors.MoveTo(this.ix);
			errmsg = errors.Error(key);
		}
		return errmsg;
	}

	public int FirstRecordOnPage() {
		return firstRec;
	}

	public String GetName() {
		return name;
	}

	public int LastRecordOnPage() {
		return lastRec;
	}

	/**
	 * Computes a URL parameter substring from the result set's primary key.
	 */
	public String
	Key()
		throws GLException
	{
		if (this.EOF())
			return "$newRecord";
		StringBuffer keynames = new StringBuffer(),
			keyvals = new StringBuffer();
		String knsep = "", kvsep = "";
		for (int i = 0; i < pkey.length; ++i) {
			Object o = Value(pkey[i]);
			if (o != null) {
				String ekey = URLEncoder.encode(pkey[i]);
				keynames.append(knsep).append(ekey);
				knsep = ",";
				keyvals.append(kvsep).append(ekey).append('=')
					.append(URLEncoder.encode(
						o.toString()));
				kvsep = "&";
			}
		}
		return keynames.append('&').append(keyvals).toString();
	}

	public Object
	Value(String key)
		throws GLException
	{
		return Value(colnum(key));
	}

	public Object
	Value(int j)
		throws GLException
	{
		/*
		 * Overlay our own value with that of the form error
		 * source (if any).
		 */
		Object o = null;
		if (errors != null) {
			errors.MoveTo(this.ix);
			o = errors.Value(this.colname[j]);
		}
		if (GLNull.val.equals(o))
			o = this.ov(j);

		if (o == null)
			return GLNull.val;
		return o;
	}
	public int AbsolutePosition() {
		return ix;
	}
	public int RecordCount() {
		return nrow;
	}

	public String[] PrimaryKey() {
		String[] a = new String[pkey.length];
		for (int i = 0; i < a.length; ++i)
			a[i] = pkey[i];
		return a;
	}

	/*
	 * -- Other public methods
	 */

	/**
	 * The JDBC type of the column named by KEY.
	 */
	public int getColumnType(String key) {
		return getColumnType(colnum(key));
	}
	/**
	 * The JDBC type of the column indexed by J.
	 */
	public int getColumnType(int j) {
		return coltype[j];
	}
	/**
	 * The database's string name for the column named by KEY.
	 */
	public String getColumnTypeName(String key) {
		return getColumnTypeName(colnum(key));
	}
	/**
	 * The database's string name for the column indexed by J.
	 */
	public String getColumnTypeName(int j) {
		return coltypename[j];
	}
	/**
	 * The name of the column at index J.
	 */
	public String getColumnName(int j) {
		return colname[j];
	}
	/**
	 * The number of columns in this result set.
	 * Columns are numbered 1..getColumnCount().
	 */
	public int getColumnCount() {
		return ncol;
	}

	public String
	toString() {
		return new StringBuffer()
			.append("<p><b>CSW ").append(name).append("</b></p>")
			.append("<table border='1' cellspacing='2'")
			.append(" cellpadding='1'>\r\n")
			.append("<tr><th>row</th><td>").append(ix)
			.append("</td></tr>\r\n")
			.append("<tr><th>source</th><td>").append(source)
			.append("</td></tr>\r\n")
			.append("<tr><th>sql</th><td>").append(sql)
			.append("</td></tr>\r\n")
			.append("<tr><th>newRecord</th><td>")
			.append(newRecord)
			.append("</td></tr>\r\n")
			.append("<tr><th>copyRecord</th><td>")
			.append(copyRecord)
			.append("</td></tr>\r\n")
			.append("<tr><th>block size</th><td>").append(blksize)
			.append("</td></tr>\r\n")
			.append("<tr><th>firstRec</th><td>").append(firstRec)
			.append("</td></tr>\r\n")
			.append("<tr><th>lastRec</th><td>").append(lastRec)
			.append("</td></tr>\r\n")
			.append("</table>\r\n").toString();
	}

	/**
	 * The following public members would have been better with
	 * access functions: <code>newRecord</code>, <code>ovthrow</code>,
	 * <code>envelope</code>, <code>indices</code>,
	 * <code>copyRecord</code>.
	 * Don't write new code that appeals to them.
	 */
	public boolean newRecord;
	public boolean ovthrow;
	public Hashtable envelope = new Hashtable();
	public ArrayList indices = new ArrayList();
	public int copyRecord = 0;

	/*
	 * -- Private
	 */
	private static final String NEW_RECORD = "$newRecord";
	private int ix;
	private int blksize;
	private int firstRec, lastRec;
	private String source, sql;
	private GLContext gl;
	private String[] pkey;
	private int nrow, ncol;
	private Object[][] a;
	private Hashtable colnums;
	private int[] coltype;
	private String[] colname;
	private String[] coltypename;
	private CSWError errors;
	private String name;

	/**
	 * Index of the column named by KEY, or 0 if none.
	 */
	private int
	colnum(String key) {
		Object o = this.colnums.get(key.toLowerCase());
		return o==null ? 0 : ((Integer) o).intValue();
	}

	/**
	 * Returns, as a comma-separated list of column names,
	 * the primary key from table TABLENAME in the database
	 * accessed by datasource DBNAME; or the empty string
	 * if none.
	 */
	private static String
	getPrimaryKey(GLContext gl, String dbname, String tablename)
		throws SQLException, GLException
	{
		Connection con = GLJdbc.getConnection(gl, dbname);
		DatabaseMetaData mdata = con.getMetaData();
		/*
		 * Bug: Oracle insists, and Postgres doesn't mind,
		 * that we coerce the table name to uppercase.
		 */
		ResultSet rs = mdata.getPrimaryKeys(null, null,
			tablename/*.toUpperCase()*/);
		Hashtable a = new Hashtable();
		while (rs.next())
			a.put(rs.getString(4), "");
		rs.close();
		con.close();
		return GLUtils.join(",", GLUtils.hkeys(a));
	}

	/**
	 * Returns a Hashtable whose keys are this's primary key
	 * column names and whose values are this's values of
	 * those columns at the current row
	 * (<code>this.AbsolutePosition()</code>).
	 */
	private Hashtable
	getWhere()
		throws GLException
	{
		Hashtable h = new Hashtable();
		for (int i = 0; i < pkey.length; ++i)
			h.put(pkey[i], Value(pkey[i]));
		return h;
	}

	/**
	 * Checks for an error object that matches this, and installs
	 * it into this if found.
	 */
	private void
	initErrors(GLContext gl) {
		CSWError cserr = gl.getError();
		if (cserr==null)
			return;
		if (this.name==null || !this.name.equals(cserr.GetName()))
			return;
		int nmatch = 0;
		for (int j = 1; j <= cserr.getColumnCount(); ++j)
			if (colnum(cserr.getColumnName(j)) == 0)
				return;
		this.errors = cserr;
	}

	/**
	 * Decodes BLOCKTYPE and the navigation parameters
	 * <code>RECORD_{KEY,INDEX}[this.GetName()]</code>
	 * to position the csw's window (<code>FirstRecordOnPage</code>,
	 * <code>BlockSize</code>).
	 */
	private void
	initPosition(String blocktype)
		throws GLException
	{
		GLBlockParam pb = new GLBlockParam(blocktype);
		if (!pb.valid)
			throw new GLException(
				"initPosition: unrecognized blocktype"
				+blocktype);
		blksize = pb.blksize==0 ? nrow : pb.blksize;

		final String rkname = "RECORD_KEY["+GetName()+"]";
		String k = gl.getRequest().getParameter(rkname);
		try {
			if (k != null)
				MoveToKey(gl.mkkey(k));
		}
		catch (GLException e) {
			gl.runtimeDebugMessage(
				GLUtils.cat("No key ", k, " in ", sql));
			MoveTo(1);
		}

		/*
		 * If we had a RECORD_INDEX request parameter, use it.
		 */
		int x = -1;
		try {
			final String rxname =
				"RECORD_INDEX["+GetName()+"]";
			x = Integer.parseInt(
				gl.getRequest().getParameter(rxname));
			if (newRecord && copyRecord == 1) {
				MoveTo(1+nrow);
				copyRecord = x;
			}
			else
				MoveTo(x < 1 ? 1 : x > nrow ? nrow : x);
		} catch (Exception e) {}

		firstRec = ix;
		lastRec = firstRec + blksize - 1;
		if (lastRec > nrow)
			lastRec = nrow;
	}

	/**
	 * The value at column J in the current row.
	 */
	private Object
	ov(int j)
		throws GLException
	{
		int i = ix == nrow+1 && newRecord && copyRecord > 0
			? copyRecord : ix;
		if (i < 1 || nrow < i || j < 1 || ncol < j) {
			if (newRecord && nrow+1==i && 1<=j && j<=ncol)
				return null;
			if (!ovthrow)	/* Baby out with bathwater? */
				return null;
			throw new GLException(
				xwhere("access out of range", j));
		}
		return a[i][j];
	}

	/**
	 * Debugging info: a nice string describing where a failed
	 * row/column access occurred.
	 */
	private String
	xwhere(String label, int j) {
		return new StringBuffer()
			.append(label).append(": ")
			.append("row ").append(ix)
			.append(", column ").append(j)
			.append(", dbname ").append(source)
			.append(", sql ").append(sql)
			.append(", newRecord ").append(newRecord)
			.append(", copyRecord ").append(copyRecord)
			.append("\r\n")
			.toString();
	}
};
