/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.jdbc;

import java.lang.*;
import java.sql.*;
import java.util.*;

/**
 * Utility class implements a simple connection pool for the benefit
 * of GLDataSource.
 */
class GLConnectionPool {
	/**
	 * Builds a connection pool with initial size NSTART,
	 * maximum size NMAX, using DS as the datasource.
	 */
	public GLConnectionPool(GLDataSource ds, int nstart, int nmax)
	{
		Long now = new Long(System.currentTimeMillis());
		this.nstart = nstart;
		this.nmax = nmax;
		this.ds = ds;
		for (int n = nstart; --n >= 0;) {
			try {
				Connection con = ds.getcon();
				avail.add(con);
				condates.put(con, now);
			}
			catch (SQLException e) {
				log(8, e.toString());
			}
		}
	}

	/**
	 * Shuts down the connection pool.
	 */
	public void
	closePool() {
		Connection con = null;
		ArrayList w = null;

		synchronized (this) {
			this.nmax = 0;
			w = avail;
			avail = null;
		}
		for (int i = w==null ? 0 : w.size(); --i >= 0;)
			closecon((Connection) w.get(i));
		condates = null;
	}

	/**
	 * Obtains a connection from the pool, or from the database
	 * if none are pooled.
	 */
	public Connection
	getConnection()
		throws java.sql.SQLException
	{
		Connection con = null;
		long drop = System.currentTimeMillis() - TIMEOUT_GONG;
		log(10, "GLConnectionPool.get {"/*}*/);
		while ((con = availcon()) != null) {
			if (condate(con) <= drop) {
				synchronized (stats) {
					++stats[STALE_BY_TIMEOUT];
				}
				log(10, "Dropped timed out connection.");
				condates.remove(con);
				con = null;
				continue;
			}

			condates.remove(con);
			try {
				con.clearWarnings();
				con.getCatalog();
			}
			catch (SQLException e) {
				synchronized (stats) {
					++stats[STALE];
				}
				con = null;
			}
			if (con != null) {
				synchronized (stats) {
					++stats[FROMPOOL];
					++stats[NOUT];
				}
				log(10, /*{*/"GLConnectionPool.get pool }");
				return con;
			}
		}

		/*
		 * Pool is empty.
		 */
		con = ds.getcon();
		synchronized (stats) {
			++stats[FROMDB];
			++stats[NOUT];
		}
		log(10, /*{*/"GLConnectionPool.get db }");
		return con;
	}

	/**
	 * The number of connections we've handed out that haven't
	 * come back yet.
	 */
	public int
	getOutstandingConnections() {
		return stats[NOUT];
	}

	/**
	 * Returns CON to the pool.
	 */
	public void
	returnConnection(Connection con) {
		if (con == null) {
			log(10, "GLConnectionPool.return"
				+ " [discard null con] {}");
			return;
		}

		int which = 0;
		log(10, "GLConnectionPool.return {"/*}*/);
		if (recyclecon(con))
			which = BACKTOPOOL;
		else {
			closecon(con);
			which = BACKTODB;
		}
		synchronized (stats) {
			++stats[which];
			--stats[NOUT];
		}
		if (logp(10))
			log(10, "GLConnectionPool.return "
				+ (which==BACKTODB ? "db" : "pool")
				+/*{*/" }");
	}

	/*
	 * -- Private.
	 */

	/**
	 * Returns a connection fetched from the pool, or null
	 * if none available.
	 */
	private synchronized Connection
	availcon() {
		if (avail != null && avail.size() > 0) {
			Connection con = (Connection) avail.get(0);
			avail.remove(0);
			return con;
		}
		return null;
	}

	/**
	 * Convenience: wraps con.close() to discard the exception.
	 */
	private void
	closecon(Connection con) {
		if (con != null)
			try {
				con.close();
			} catch (SQLException e) {}
	}

	private long
	condate(Connection con) {
		Long l = (Long) condates.get(con);
		log(10, "condate "+l);
		return l==null ? 0 : l.longValue();
	}

	private void
	log(int thresh, String msg) {
		if (logLevel >= thresh)
			System.out.println(msg);
	}

	private boolean
	logp(int thresh) {
		return logLevel >= thresh;
	}

	/**
	 * Returns TRUE iff it can put CON back into the pool
	 * (caller must decide what to do if FALSE: typically,
	 * close the connection itself).
	 */
	private synchronized boolean
	recyclecon(Connection con) {
		if (avail != null && avail.size() < this.nmax) {
			avail.add(con);
			condates.put(con, new Long(System.currentTimeMillis()));
			return true;
		}
		return false;
	}

	public void
	getStatistics(StringBuffer ts) {
		int m0, stale, frompool, fromdb, backtopool, backtodb, nout;
		int stale_by_timeout;
		synchronized (stats) {
			m0 = stats[0];
			stale = stats[STALE];
			stale_by_timeout = stats[STALE_BY_TIMEOUT];
			frompool = stats[FROMPOOL];
			fromdb = stats[FROMDB];
			backtopool = stats[BACKTOPOOL];
			backtodb = stats[BACKTODB];
			nout = stats[NOUT];
		}
		ts.append("frompool ").append(frompool)
			.append("\nfromdb ").append(fromdb)
			.append("\nbacktopool ").append(backtopool)
			.append("\nbacktodb ").append(backtodb)
			.append("\noutstanding ").append(nout)
			.append("\nstale ").append(stale)
			.append("\nstale_by_timeout ").append(stale_by_timeout)
			.append("\nbucket0 ").append(m0);
	}

	public static final int STALE = 1,
		FROMPOOL = 2, FROMDB = 3,
		BACKTOPOOL = 4, BACKTODB = 5,
		NOUT = 6,
		STALE_BY_TIMEOUT = 7,
		NSTATS = 8;
	/**
	 * Data source provides the real connections.
	 */
	private GLDataSource ds;
	/**
	 * The pool of connections available for use.
	 * NB ArrayList methods are not synchronized; synchronize
	 * explicitly on `this' instead.
	 *
	 * After closePool(), avail==null.
	 */
	private ArrayList avail = new ArrayList();
	/**
	 * NFIRST, NMORE, NMAX are pool size parameters.
	 * The default values provide unpooled connections.
	 */
	private int nstart = 0, nmore = 1, nmax = 0;
	/**
	 * log(N, MSG) prints MSG if N <= logLevel.
	 */
	private static final int logLevel = 9;
	/**
	 * Statistics.
	 */
	private int[] stats = new int[NSTATS];
	/**
	 * Expire old connections on general principles.
	 */
	private Hashtable condates = new Hashtable();
	private static final long TIMEOUT_GONG = 45 * 60 * 1000;
}
