/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import org.apache.regexp.*;

/**
 * General text filters.
 * This class only exists to provide a name space.
 *
 * We pass the text to be filtered as Object, rather than as String,
 * for the benefit of GoLive Dynamic Content -- it is fishing out of
 * datasources things that might require conversion.
 */
public abstract class GLFilters {
	/**
	 * convertLineEndings filter.
	 * <p>Converts text to preserve the original's line breaks and
	 * paragraph boundaries (inferred from the presence of two
	 * consecutive line breaks) in HTML output.
	 * <p>The encodeHTML parameter helps work around a GL limitation;
	 * if we could compose filters, we could say<pre>
	 *	GLFilters.convertLineEndings(gl, GLUtils.htmlEncode(text),
	 *		useBR, useP);
	 * </pre>
	 */
	public static String
	convertLineEndings(GLContext gl, Object texto, boolean useNBSP,
		boolean useBR, boolean useP, boolean encodeHTML)
		throws GLException
	{
		String text = GLNull.nonnull(texto).toString();
		RE patCRLF = null, pat2 = null, pat1 = null;

		try {
			if (useP || useBR)
				patCRLF = new RE(str_CRLF, RE.MATCH_MULTILINE);
			if (useP)
				pat2 = new RE(str_2LF, RE.MATCH_MULTILINE);
			if (useBR)
				pat1 = new RE(str_1LF, RE.MATCH_MULTILINE);
		}
		catch (RESyntaxException e) {
			throw new GLException(e.toString());
		}

		if (encodeHTML)
			text = GLUtils.htmlEncode(text);
		/*
		 * Convert platform linebreaks into a special internal form.
		 */
		if (useP || useBR) {
			if (patCRLF.match(text))
				text = patCRLF.subst(text, str_1LF);
		}
		/* <P> for double newlines */
		if (useP) {
			if (pat2.match(text))
				text = pat2.subst(text, "\r\n<P>");
		}
		/* <BR> for singles */
		if (useBR) {
			if (pat1.match(text))
				text = pat1.subst(text, "<BR>\r\n");
		}
		if (useNBSP)
			text = nbsp(text);
		return text;
	}

	/**
	 * Throw away GL -- this version for code generated by GoLive
	 * which needs a stylized parameter list for functions it calls.
	 * @see com.adobe.gl.runtime6.utils.GLUtils#htmlEncode
	 */
	public static String
	htmlEncode(GLContext gl, Object os) {
		return GLUtils.htmlEncode(os);
	}

	/**
	 * This entrypoint is only for GoLive's convenience in writing code.
	 * @see com.adobe.gl.runtime6.utils.GLContext#mapPath
	 */
	public static String
	mapPath(GLContext gl, Object o) {
		return gl.mapPath(o);
	}

	/**
	 * This entrypoint is only for GoLive's convenience in writing code.
	 * @see java.net.URLEncoder#encode
	 */
	public static String
	urlEncode(GLContext gl, Object os) {
		return URLEncoder.encode(GLNull.nonnull(os).toString());
	}

	public static final boolean USE_NBSP = true, USE_BR = true,
		USE_P = true, USE_ENCODE = true;

	/*
	 * Private.
	 */

	private static String
	nbsp(String s)
	{
		StringBuffer ts = new StringBuffer();
		final int N = s.length();
		final int SP_PER_TAB = 4;
		int runlen = 0;
		for (int i = 0; i < N; ++i) {
			final char c = s.charAt(i);
			if (c==' ')
				++runlen;
			else if (c=='\t')
				runlen += SP_PER_TAB;
			else {
				if (runlen > 0) nbspAdd(ts, runlen);
				runlen = 0;
				ts.append(c);
			}
		}
		if (runlen > 0) nbspAdd(ts, runlen);
		return ts.toString();
	}
	private static void
	nbspAdd(StringBuffer ts, int n) {
		while (--n > 0)
			ts.append("&nbsp;");
		ts.append(' ');
	}

	private static final String str_CRLF = "\r\n?|\n",
		str_1LF = "\1", str_2LF = "\1\1";
};
