/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.lang.*;
import java.net.*;
import java.text.*;
import java.util.*;

/**
 * Number formatting routines.
 * Each calls the appropriate java.text.NumberFormat.get*Instance()
 * routine and then applies the parameters to control minimum fractional
 * digits, maximum fractional digits, minimum integer digits, and an
 * indication of whether digits are grouped.
 */
public class GLNumberFormat {
	/**
	 * Constructor is private because I don't think this ever
	 * need be instantiated.
	 */
	private GLNumberFormat() {
	}

	/*
	 * I wonder if we should cache the get*Instance() values...
	 */

	/**
	 * Format O, which should be a <code>java.lang.Number</code> instance,
	 * as a currency value in the locale derived from GL.  Use at least
	 * MINFRACT fractional digits, at most MAXFRACT digits, and at
	 * least LEADING digits.  If any of these are -1, use the locale's
	 * default.
	 * GROUPDIGITS tells whether to group digits with commas (or the
	 * locale-specific group separator): if it's USETRUE, do;
	 * USEFALSE, don't; USEDEFAULT, follow the locale's lead.
	 */
	public static String
	Currency(GLContext gl, Object o,
		int minFract, int maxFract, int leading, int groupDigits)
	{
		NumberFormat nf =
			NumberFormat.getCurrencyInstance(gl.getUserLocale());
		fixup(nf, minFract, maxFract, leading, groupDigits);
		return format(gl, nf, o);
	}
	/**
	 * Format O, which should be a <code>java.lang.Number</code> instance,
	 * as a number in the locale derived from GL.  Use at least
	 * MINFRACT fractional digits, at most MAXFRACT digits, and at
	 * least LEADING digits.  If any of these are -1, use the locale's
	 * default.
	 * GROUPDIGITS tells whether to group digits with commas (or the
	 * locale-specific group separator): if it's USETRUE, do;
	 * USEFALSE, don't; USEDEFAULT, follow the locale's lead.
	 * least LEADING digits.  GROUPDIGITS tells whether to group digits
	 * with commas (or locale-specific group separator): if it's
	 * USETRUE, do; USEFALSE, don't; USEDEFAULT, follow the locale's lead.
	 */
	public static String
	Number(GLContext gl, Object o,
		int minFract, int maxFract, int leading, int groupDigits)
	{
		NumberFormat nf =
			NumberFormat.getInstance(gl.getUserLocale());
		fixup(nf, minFract, maxFract, leading, groupDigits);
		return format(gl, nf, o);
	}
	/**
	 * Format O, which should be a <code>java.lang.Number</code> instance,
	 * as a percentage in the locale derived from GL.  Use at least
	 * MINFRACT fractional digits, at most MAXFRACT digits, and at
	 * least LEADING digits.  If any of these are -1, use the locale's
	 * default.
	 * GROUPDIGITS tells whether to group digits with commas (or the
	 * locale-specific group separator): if it's USETRUE, do;
	 * USEFALSE, don't; USEDEFAULT, follow the locale's lead.
	 */
	public static String
	Percent(GLContext gl, Object o,
		int minFract, int maxFract, int leading, int groupDigits)
	{
		NumberFormat nf =
			NumberFormat.getPercentInstance(gl.getUserLocale());
		fixup(nf, minFract, maxFract, leading, groupDigits);
		return format(gl, nf, o);
	}

	public static final int useDefault = -1,
		useFalse = 0, useTrue = 1;

	private static void
	fixup(NumberFormat nf, int minFract, int maxFract, int leading,
		int groupDigits)
	{
		if (minFract > 0)
			nf.setMinimumFractionDigits(minFract);
		if (maxFract > 0)
			nf.setMaximumFractionDigits(maxFract);
		if (leading != useDefault)
			nf.setMinimumIntegerDigits(leading);
		if (groupDigits != useDefault)
			nf.setGroupingUsed(groupDigits != useFalse);
	}

	/**
	 * Formats O according to NF, and returns the formatted number
	 * on success, or O.toString() on failure.
	 * O might not be a Number -- attempts to recognize it by type,
	 * or by parsing O.toString().
	 */
	private static String
	format(GLContext gl, NumberFormat nf, Object o) {
		double t = -42.0;
	set: {
		if (o instanceof Number) {
			t = ((Number) o).doubleValue();
			break set;
		}
		if (o instanceof Boolean) {
			t = ((Boolean) o).booleanValue() ? 1.0 : 0.0;
			break set;
		}
		String s = GLNull.nonnull(o).toString();
		if ("".equals(s)) {
			t = 0;
			break set;
		}
		try {
			t = nf.parse(s).doubleValue();
			break set;
		}
		catch (ParseException e) {
			try {
				t = NumberFormat.getInstance(gl.getUserLocale())
					.parse(s).doubleValue();
				break set;
			}
			catch (ParseException e2) {
				gl.runtimeDebugMessage(e2.toString());
			}
		}
		return s;
		}

		return nf.format(t);
	}
};
