/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.util.*;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.adobe.gl.runtime6.xml.GLXML;

/**
 * Reads application parameters not stored in <code>WEB-INF/web.xml</code>.
 *
 * Some of the parameters for a GoLive Dynamic Content web application,
 * for example, the "friends" data, want to be managed as separate files
 * rather than as init parameters in WEB-INF/web.xml.
 * Here we read an XML file that looks like
 *	&lt;top-level>
 *	&lt;table>
 *	&lt;tr>
 *	&lt;td>v&lt;/value> ...
 *	&lt;/tr> ...
 *	&lt;/table>
 *	&lt;/top-level>
 * and return the values as a two-dimensional table of strings.
 */
public class GLTableReader {
	private GLTableReader(GLContext gl) {
		this.gl = gl;
	}

	/**
	 * Read the parameters from file FILE.
	 * Return a two-dimensional table of string values.
	 */
	public static String[][]
	getData(GLContext gl, File file)
		throws GLException
	{
		Document document = null;
		try {
			document = GLXML.asDom(gl, file);
		}
		catch (IOException e) {
			throw new GLException(e.toString());
		}

		GLTableReader rdr = new GLTableReader(gl);
		rdr.fill(document);
		return rdr.result;
	}

	private void fill(Node node) {
		if (node==null)
			return;
		switch (node.getNodeType()) {
		case Node.DOCUMENT_NODE:
			fill(((Document) node).getDocumentElement());
			break;
		case Node.ELEMENT_NODE:
			stk.push(node.getNodeName());
			NodeList children = node.getChildNodes();
			int nc = children==null ? 0 : children.getLength();
			for (int i = 0; i < nc; ++i)
				fill(children.item(i));
			if (stackHas(table_tr)) {
				String[] a = new String[v.size()];
				v.copyInto(a);
				w.add(a);
				v.setSize(0);
			}
			if (stackHas(table)) {
				String[][] a = new String[w.size()][];
				w.copyInto(a);
				result = a;
				w.setSize(0);
			}
			stk.pop();
			break;
		case Node.TEXT_NODE:
			if (stackHas(table_tr_td))
				v.add(node.getNodeValue());
			break;
		}
	}

	private boolean stackHas(String[] a) {
		int nx = stk.size()-a.length;
		if (nx < 0)
			return false;
		for (int i = 0; i < a.length; ++i)
			if (!a[i].equals(stk.get(nx+i)))
				return false;
		return true;
	}

	private static final String[] table_tr_td = { "table", "tr", "td" };
	private static final String[] table_tr = { "table", "tr" };
	private static final String[] table = { "table" };

	private Stack stk = new Stack();
	private Vector v = new Vector(), w = new Vector();
	private String[][] result = null;
	private GLContext gl;
};
