/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import javax.servlet.jsp.*;

/**
 * Subroutines for writing XML files to a JspWriter.
 * In method parameter lists, OUT is the JspWriter,
 * and DEPTH is the number of tabs to use when indenting output.
 */
public class GLXOut {
	private GLXOut() {}

	public static void
	crlf(JspWriter out)
		throws IOException
	{
		out.print("\r\n");
	}

	public static String
	dummy(Object s) {
		return dummy(s, "");
	}

	public static String
	dummy(Object s, String defval) {
		return s!=null ? s.toString() : defval;
	}

	/**
	 * Writes <code>
	 * &lt;DATASOURCES<br>&gt;</code>
	 * and <code>
	 * &lt;DATASOURCE&gt;names[i]&lt;/DATASOURCE&gt;</code>
	 * for the datasource NAMES.
	 */
	public static void
	dumpDataSources(JspWriter out, String[] names)
		throws IOException
	{
		stag(out, "DATASOURCES", 0);
		for (int i = 0; i < names.length; ++i)
			oneElement(out, "DATASOURCE", xmlEncode(names[i]), 1);
		etag(out, "DATASOURCES", 0);
	}

	/**
	 * &lt;ERROR Number='ERRORCODE' Description='DESC' />
	 */
	public static void
	errorMessage(JspWriter out, String errorCode, String desc)
		throws IOException
	{
		out.print("<ERROR Number='");
		out.print(xmlEncode(errorCode));
		out.print("' ");
		out.print("Description='");
		out.print(xmlEncode(desc));
		out.print("' ");
		out.println("/>");
	}

	/**
	 * An end tag for the element named ELEM.
	 */
	public static void
	etag(JspWriter out, String elem, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print("</"); out.print(tagOnly(elem)); out.print('>');
		crlf(out);
	}

	public static void
	indent(JspWriter out, int depth)
		throws IOException
	{
		while (--depth >= 0)
			out.print('\t');
	}

	/**
	 * Writes a header XML comment containing TEXT.
	 * Removes '--' pairs from text (replaces with escaped values)
	 * to prevent lexical errors by picky XML processors.
	 * Trailing crlf.
	 */
	public static void
	headerComment(JspWriter out, String text, int depth)
		throws IOException
	{
		indent(out, depth);
		out.println("<!--");
		indent(out, 1+depth);
		out.println(undash(text));
		indent(out, depth);
		out.println("  -->");
	}

	/**
	 * Writes an inline XML comment containing TEXT.
	 * Removes '--' pairs from text (replaces with escaped values)
	 * to prevent lexical errors by picky XML processors.
	 * No trailing crlf.
	 */
	public static void
	inlineComment(JspWriter out, String text, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print("<!-- ");
		out.print(undash(text));
		out.print(" -->");
	}

	/**
	 * Print &lt;FIELD>VALUE&lt;/FIELD>\n to OUT.
	 */
	public static void
	oneElement(JspWriter out, String field, Object value, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print('<'); out.print(field); out.print('>');
		out.print(value);
		/* For close tag, strip off attributes if any */
		out.print("</"); out.print(tagOnly(field)); out.print('>');
		crlf(out);
	}

	public static void
	oneMessage(JspWriter out, String msg)
		throws IOException
	{
		oneElement(out, "MESSAGE", xmlEncode(msg), 1);
	}

	/**
	 * The start tag for an element named ELEM.
	 */
	public static void
	stag(JspWriter out, String elem, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print('<'); out.print(elem); out.print('>');
		crlf(out);
	}

	/**
	 * Just the tag (no attributes).
	 */
	public static String
	tagOnly(String tag) {
		final int N = tag.length();
		for (int i = 0; i < N; ++i)
			if (Character.isWhitespace(tag.charAt(i)))
				return tag.substring(0, i);
		return tag;
	}

	/**
	 * Convert S into a string safe to appear within an XML comment
	 * (convert -&#45; into -&amp;#45; but don't disturb other chars).
	 */
	public static String
	undash(String s) {
		if (s.indexOf("--") < 0)
			return s;
		return GLUtils.replace(s, "--", "-&#45;");
	}

	/**
	 * Convert O into a string safe to appear as XML data
	 * by converting the syntax characters into character
	 * references.
	 */
	public static String
	xmlEncode(Object o) {
		String s = dummy(o, "");
		final int n = s.length();

	findxchar:
		{
		for (int i = 0; i < n; ++i) {
			final char c = s.charAt(i);
			if (c=='<' || c=='>' || c=='&' || c=='"' || c=='\'')
				break findxchar;
		}
		return s;
		}

		StringBuffer ts = new StringBuffer();
		for (int i = 0; i < n; ++i) {
			final char c = s.charAt(i);
			switch (c)
			{
			case '<': ts.append("&lt;"); break;
			case '>': ts.append("&gt;"); break;
			case '&': ts.append("&amp;"); break;
			case '"': ts.append("&quot;"); break;
			case '\'': ts.append("&apos;"); break;
			default: ts.append(c);
			}
		}
		return ts.toString();
	}

	public static final boolean SINGLETON = true;
}
